Package: cAIC4
Type: Package
Title: Conditional Akaike Information Criterion for 'lme4' and 'nlme'
Version: 1.1
Date: 2025-04-04
Authors@R: c(person(given = "Benjamin", family = "Saefken", role = "aut"),
 person(given = "David", family = "Ruegamer", role = c("aut", "cre"), email = "david.ruegamer@gmail.com"),
 person(given = "Philipp", family = "Baumann", role = "aut"),
 person(given = "Rene-Marcel", family = "Kruse", role = "aut"),
 person(given = "Sonja", family = "Greven", role = "aut"),
 person(given = "Thomas", family = "Kneib", role = "aut"))
Depends: lme4(>= 1.1-6), methods, Matrix, stats4, nlme
Imports: RLRsim, mgcv, mvtnorm
Suggests: gamm4
Description: Provides functions for the estimation of the conditional Akaike
    information in generalized mixed-effect models fitted with (g)lmer() 
    from 'lme4', lme() from 'nlme' and gamm() from 'mgcv'.
    For a manual on how to use 'cAIC4', see Saefken et al. (2021) <doi:10.18637/jss.v099.i08>.
License: GPL (>= 2)
Packaged: 2025-04-03 23:29:48 UTC; david
NeedsCompilation: no
Date/Publication: 2025-04-04 00:10:02 UTC
RoxygenNote: 7.3.2
Encoding: UTF-8
Author: Benjamin Saefken [aut],
  David Ruegamer [aut, cre],
  Philipp Baumann [aut],
  Rene-Marcel Kruse [aut],
  Sonja Greven [aut],
  Thomas Kneib [aut]
Maintainer: David Ruegamer <david.ruegamer@gmail.com>
Repository: CRAN
Built: R 4.6.0; ; 2026-04-26 11:58:09 UTC; unix
