# Generated by roxygen2: do not edit by hand


if(getRversion() >= "3.3.0") {
importFrom("stats", sigma)
} else {
importFrom("lme4", sigma)
}
S3method(deleteZeroComponents,lme)
S3method(deleteZeroComponents,merMod)
S3method(family,lme)
S3method(getModelComponents,lme)
S3method(getModelComponents,merMod)
S3method(getcondLL,lme)
S3method(getcondLL,merMod)
S3method(print,cAIC)
export(anocAIC)
export(cAIC)
export(deleteZeroComponents)
export(getModelComponents)
export(getWeights)
export(getcondLL)
export(modelAvg)
export(predictMA)
export(stepcAIC)
export(summaryMA)
import(Matrix)
import(RLRsim)
import(lme4)
import(methods)
import(mvtnorm)
import(parallel)
importFrom(mgcv,gamm)
importFrom(nlme,getResponse)
importFrom(nlme,pdDiag)
importFrom(stats,as.formula)
importFrom(stats,dbinom)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,reformulate)
importFrom(stats,residuals)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(stats,weights)
importFrom(stats4,logLik)
importFrom(utils,capture.output)
importFrom(utils,combn)
