#' Data from Gu and Wahba (1991)
#'
#' @name guWahbaData
#' @description Data from Gu and Wahba (1991) which is used for demonstrative purposes to exemplarily fit 
#' a generalized additive mixed model.
#' @docType data
#' @references Gu and Wahba (1991) Minimizing GCV/GML scores with multiple smoothing parameters 
#' via the Newton method. SIAM J. Sci. Statist. Comput. 12:383-398
#' @keywords data
NULL
#' Subset of the Zambia data set on childhood malnutrition
#' 
#' @name Zambia
#' @description Data analyzed by Kandala et al. (2001) which is used for demonstrative purposes to 
#' estimate linear mixed and additive models using a stepwise procedure on the basis of the cAIC. 
#' The full data set is 
#' available at \url{https://www.uni-goettingen.de/de/551625.html}.
#' @docType data
#' @references Kandala, N. B., Lang, S., Klasen, S., Fahrmeir, L. (2001): 
#' Semiparametric Analysis of the Socio-Demographic and Spatial 
#' Determinants of Undernutrition in Two African Countries. 
#' Research in Official Statistics, 1, 81-100.
#' @keywords data
NULL